#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
# University of Wisconsin-Madison.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
#     input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""The Arctic Weather Satellite-1 (AWS-1) MicroWave Radiometer (MWR) reader
is for reading L1B files for the MWR instrument.

These files are NetCDF4 files. The reader can be specified with
the ``polar2grid.sh`` command using the ``aws1_mwr_l1b_nc`` reader name.

This reader's default resampling algorithm is ``ewa`` for Elliptical Weighted
Averaging resampling. The ``--weight-delta-max`` option is set to 100 and
``--weight-distance-max`` is set to 1.

The MWR reader provides the following products:

+--------------------+--------------------------------------------+
| Product Name       | Description                                |
+====================+============================================+
| bt01               | Band 1 (50.3GHz) Brightness Temperature    |
+--------------------+--------------------------------------------+
| bt02               | Band 2 (52.8GHz) Brightness Temperature    |
+--------------------+--------------------------------------------+
| bt03               | Band 3 (53.2GHz) Brightness Temperature    |
+--------------------+--------------------------------------------+
| bt04               | Band 4 (53.6GHz) Brightness Temperature    |
+--------------------+--------------------------------------------+
| bt05               | Band 5 (54.4GHz) Brightness Temperature    |
+--------------------+--------------------------------------------+
| bt06               | Band 6 (54.9GHz) Brightness Temperature    |
+--------------------+--------------------------------------------+
| bt07               | Band 7 (55.5GHz) Brightness Temperature    |
+--------------------+--------------------------------------------+
| bt08               | Band 8 (57.3GHz) Brightness Temperature    |
+--------------------+--------------------------------------------+
| bt09               | Band 9 (89.0GHz) Brightness Temperature    |
+--------------------+--------------------------------------------+
| bt10               | Band 10 (165.5GHz) Brightness Temperature  |
+--------------------+--------------------------------------------+
| bt11               | Band 11 (176.3GHz) Brightness Temperature  |
+--------------------+--------------------------------------------+
| bt12               | Band 12 (178.8GHz) Brightness Temperature  |
+--------------------+--------------------------------------------+
| bt13               | Band 13 (180.3GHz) Brightness Temperature  |
+--------------------+--------------------------------------------+
| bt14               | Band 14 (181.5GHz) Brightness Temperature  |
+--------------------+--------------------------------------------+
| bt15               | Band 15 (182.3GHz) Brightness Temperature  |
+--------------------+--------------------------------------------+
| bt16               | Band 16 (325.2GHz) Brightness Temperature  |
+--------------------+--------------------------------------------+
| bt17               | Band 17 (325.2GHz) Brightness Temperature  |
+--------------------+--------------------------------------------+
| bt18               | Band 18 (325.2GHz) Brightness Temperature  |
+--------------------+--------------------------------------------+
| bt19               | Band 19 (325.2GHz) Brightness Temperature  |
+--------------------+--------------------------------------------+

"""

from __future__ import annotations

from argparse import ArgumentParser, _ArgumentGroup
import logging
from typing import Optional

from satpy import DataQuery, Scene

from ._base import ReaderProxyBase

logger = logging.getLogger(__name__)

FILTERS = {}

PRODUCT_ALIASES = {f"bt{band_num:02d}": DataQuery(name=str(band_num)) for band_num in range(1, 20)}
BT_BANDS = sorted(PRODUCT_ALIASES.keys())


class ReaderProxy(ReaderProxyBase):
    """Provide Polar2Grid-specific information about this reader's products."""

    is_polar2grid_reader = True

    def get_default_products(self) -> list[str]:
        """Get products to load if users hasn't specified any others."""
        return BT_BANDS

    def get_all_products(self) -> list[str]:
        """Get all polar2grid products that could be loaded."""
        return BT_BANDS

    @property
    def _aliases(self) -> dict[str, DataQuery]:
        return PRODUCT_ALIASES


def add_reader_argument_groups(
    parser: ArgumentParser, group: Optional[_ArgumentGroup] = None
) -> tuple[Optional[_ArgumentGroup], Optional[_ArgumentGroup]]:
    """Add reader-specific command line arguments to an existing argument parser.

    If ``group`` is provided then arguments are added to this group. If not,
    a new group is added to the parser and arguments added to this new group.

    """
    if group is None:
        group = parser.add_argument_group(title="AWS1 MWR L1b Reader")
    return group, None



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2014 Space Science and Engineering Center (SSEC),
# University of Wisconsin-Madison.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
#     input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Functions that handle polar2grid data types."""

import logging

import numpy as np

log = logging.getLogger(__name__)

NUMPY_DTYPE_STRS = [
    "uint8",
    "uint16",
    "uint32",
    "uint64",
    "int8",
    "int16",
    "int32",
    "int64",
    "float32",
    "float64",
]


def str_to_dtype(dtype_str: str) -> np.dtype:
    try:
        return getattr(np, dtype_str)
    except AttributeError:
        raise ValueError("Not a valid data type string: %s" % (dtype_str,)) from err


def dtype_to_str(numpy_dtype: np.dtype | str) -> str:
    if isinstance(numpy_dtype, str) or not hasattr(numpy_dtype, "name"):
        numpy_dtype = np.dtype(numpy_dtype)

    try:
        return numpy_dtype.name
    except KeyError as err:
        raise ValueError("Unsupported numpy data type: %r" % (numpy_dtype,)) from err


def clip_to_data_type(data, data_type):
    if np.issubdtype(np.dtype(data_type), np.floating):
        return data
    info = np.iinfo(data_type)
    rmin = info.min
    rmax = info.max
    log.debug("Clipping data to a %d - %d data range" % (rmin, rmax))
    return data.clip(rmin, rmax)


def int_or_float(val):
    """Convert a string to integer or float.

    If we can't make it an integer then make it a float.

    """
    try:
        return int(val)
    except ValueError:
        return float(val)



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2014 Space Science and Engineering Center (SSEC),
# University of Wisconsin-Madison.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
#     input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
#
# Written by David Hoese    September 2014
# University of Wisconsin-Madison
# Space Science and Engineering Center
# 1225 West Dayton Street
# Madison, WI  53706
# david.hoese@ssec.wisc.edu
"""Helper functions and classes used by multiple polar2grid scripts.

:author:       David Hoese (davidh)
:contact:      david.hoese@ssec.wisc.edu
:organization: Space Science and Engineering Center (SSEC)
:copyright:    Copyright (c) 2013 University of Wisconsin SSEC. All rights reserved.
:date:         Sept 2014
:license:      GNU GPLv3


"""

__docformat__ = "restructuredtext en"

import argparse
import logging
import os
import sys
from collections import defaultdict

LOG = logging.getLogger(__name__)


# class SatPyWarningFilter(logging.Formatter):
#     def format(self, record):
#         if record.name.startswith('satpy') and record.levelno == logging.WARNING:
#             record.levelno = logging.DEBUG
#             record.levelname = 'DEBUG'
#         return super(SatPyWarningFilter, self).format(record)


class SatPyWarningFilter(logging.Filter):
    def filter(self, record):
        is_satpy = record.name.startswith("satpy")
        is_msg = "The following datasets were not created and may require resampling" in record.msg
        return 0 if is_satpy and is_msg else 1


class ThirdPartyFilter(logging.Filter):
    def __init__(self, ignored_packages, level=logging.WARNING, name=""):
        self.ignored_packages = ignored_packages
        self.level_filter = level
        super(ThirdPartyFilter, self).__init__(name)

    def filter(self, record):
        for pkg in self.ignored_packages:
            if record.name.startswith(pkg) and record.levelno < self.level_filter:
                return 0
        return 1


def setup_logging(console_level=logging.INFO, log_filename="polar2grid.log", log_numpy=True):
    """Set up the logger to the console to the logging level defined in the command line (default INFO).

    Sets up a file logging for everything,
    regardless of command line level specified.  Adds extra logger for
    tracebacks to go to the log file if the exception is caught.  See
    `exc_handler` for more information.

    :param console_level: Python logging level integer (ex. logging.INFO).
    :param log_filename: Log messages to console and specified log_filename (None for no file log)
    :param log_numpy: Tell numpy to log invalid values encountered
    """
    # set the root logger to DEBUG so that handlers can have all possible messages to filter
    root_logger = logging.getLogger("")
    root_logger.setLevel(min(console_level, logging.DEBUG))

    # Console output is minimal
    console = logging.StreamHandler(sys.stderr)
    console_format = "%(levelname)-8s : %(message)s"
    console.setFormatter(logging.Formatter(console_format))
    console.setLevel(console_level)
    console.addFilter(SatPyWarningFilter())
    if console_level > logging.DEBUG:
        # if we are only showing INFO/WARNING/ERROR messages for P2G then
        # filter out messages from these packages
        console.addFilter(
            ThirdPartyFilter(["satpy", "pyresample", "pyspectral", "trollimage", "pyorbital", "trollsift"])
        )
    root_logger.addHandler(console)

    # Log file messages have a lot more information
    if log_filename:
        file_handler = logging.FileHandler(log_filename)
        file_format = "[%(asctime)s] : %(levelname)-8s : %(name)s : %(funcName)s : %(message)s"
        file_handler.setFormatter(logging.Formatter(file_format))
        file_handler.setLevel(logging.DEBUG)
        root_logger.addHandler(file_handler)

        # Make a traceback logger specifically for adding tracebacks to log file
        traceback_log = logging.getLogger("traceback")
        traceback_log.propagate = False
        traceback_log.setLevel(logging.ERROR)
        traceback_log.addHandler(file_handler)

    if log_numpy:
        import numpy

        class TempLog(object):
            def write(self, msg):
                logging.getLogger("numpy").debug(msg)

        numpy.seterr(invalid="log")
        numpy.seterrcall(TempLog())


def rename_log_file(new_filename):
    """Rename the file handler for the root logger and the traceback logger."""
    # the traceback logger only has 1 handler so let's get that
    traceback_log = logging.getLogger("traceback")
    if not traceback_log.handlers:
        LOG.error("Tried to change the log filename, but no log file was configured")
        raise RuntimeError("Tried to change the log filename, but no log file was configured")

    h = traceback_log.handlers[0]
    fn = h.baseFilename
    h.stream.close()
    root_logger = logging.getLogger("")
    root_logger.removeHandler(h)
    traceback_log.removeHandler(h)

    # move the old file
    if os.path.isfile(new_filename):
        with open(new_filename, "a") as new_file:
            with open(fn, "r") as old_file:
                new_file.write(old_file.read())
        os.remove(fn)
    else:
        os.rename(fn, new_filename)

    # create the new handler
    file_handler = logging.FileHandler(new_filename)
    file_format = "[%(asctime)s] : PID %(process)6d : %(levelname)-8s : %(name)s : %(funcName)s : %(message)s"
    file_handler.setFormatter(logging.Formatter(file_format))
    file_handler.setLevel(logging.DEBUG)
    root_logger.addHandler(file_handler)
    traceback_log.addHandler(file_handler)

    LOG.debug("Log renamed from '%s' to '%s'", fn, new_filename)


def create_exc_handler(glue_name):
    def exc_handler(exc_type, exc_value, traceback):
        """Handle logging/printing an exception if it occurs.

        This will save us from print tracebacks or unrecognizable errors to the user's console.

        Note, however, that this doesn't effect code in a separate process as the
        exception never gets raised in the parent.
        """
        logging.getLogger(glue_name).error(
            "Unexpected error. Enable debug messages (-vvv) or see log file for details."
        )
        logging.getLogger(glue_name).debug("Unexpected error exception: ", exc_info=(exc_type, exc_value, traceback))
        tb_log = logging.getLogger("traceback")
        if tb_log.handlers:
            tb_log.error(exc_value, exc_info=(exc_type, exc_value, traceback))

    return exc_handler


class NumpyDtypeList(list):
    """Magic list to allow dtype objects to match string versions of themselves."""

    def __contains__(self, item):
        if super(NumpyDtypeList, self).__contains__(item):
            return True
        try:
            return super(NumpyDtypeList, self).__contains__(item().dtype.name)
        except AttributeError:
            return False


class ExtendAction(argparse.Action):
    """Similar to the 'append' action, but expects multiple elements instead of just one."""

    def __call__(self, parser, namespace, values, option_string=None):
        current_values = getattr(namespace, self.dest, []) or []
        current_values.extend(values)
        setattr(namespace, self.dest, current_values)


class ExtendConstAction(argparse.Action):
    """Similar to the 'append' action, but expects multiple elements instead of just one."""

    def __init__(self, option_strings, dest, nargs=0, **kwargs):
        if nargs:
            raise ValueError("nargs is not allowed")
        super(ExtendConstAction, self).__init__(option_strings, dest, nargs=0, **kwargs)

    def __call__(self, parser, namespace, values, option_string=None):
        current_values = getattr(namespace, self.dest, []) or []
        current_values.extend(self.const)
        setattr(namespace, self.dest, current_values)


class BooleanFilterAction(argparse.BooleanOptionalAction):
    """Action to add or not add a filter string to a list of filters."""

    def __init__(self, option_strings, dest, const: str, **kwargs):
        super().__init__(option_strings, dest, **kwargs)
        self.const = const

    def __call__(self, parser, namespace, values, option_string=None):
        if option_string in self.option_strings:
            prev = getattr(namespace, self.dest, None)
            if prev is None:
                prev = []
                setattr(namespace, self.dest, prev)

            should_include = not option_string.startswith("--no-")
            has_const = self.const in prev
            # modify list inplace
            if should_include and not has_const:
                prev.append(self.const)
            elif not should_include and has_const:
                prev.remove(self.const)


class ArgumentParser(argparse.ArgumentParser):
    def _get_group_actions(self, group):
        """Get all the options/actions in a group including those from subgroups of the provided group.

        .. note::

            This does not group the subgroup options as their own dictionaries.

        """
        these_actions = [action for action in group._group_actions]
        # get actions if this group has even more subgroups
        for subgroup in group._action_groups:
            these_actions += self._get_group_actions(subgroup)

        return these_actions

    def parse_args(self, *args, **kwargs):
        """Parse arguments to support custom polar2grid 'subgroup' behavior.

        :param subgroup_titles: Groups and their arguments that will be put
                                in a separate dictionary in the 'subgroup_args' attribute
        :param global_keywords: Keywords/arguments that should be added to all subgroup dictionaries
        """
        subgroup_titles = kwargs.pop("subgroup_titles", [])
        global_keywords = kwargs.pop("global_keywords", [])
        args = super(ArgumentParser, self).parse_args(*args, **kwargs)
        args.global_kwargs = {kw: getattr(args, kw) for kw in global_keywords}
        # a dictionary that holds arguments from the specified subgroups (defaultdict for easier user by caller)
        args.subgroup_args = defaultdict(dict)
        for subgroup_title in subgroup_titles:
            try:
                subgroup = [x for x in self._action_groups if x.title == subgroup_title][0]
            except IndexError:
                # we don't have any loggers configured at this point
                print("WARNING: Couldn't find argument group '%s' in configured parser" % (subgroup_title,))
                continue

            subgroup_args = {}
            for action in self._get_group_actions(subgroup):
                if hasattr(args, action.dest):
                    subgroup_args[action.dest] = getattr(args, action.dest)
                    delattr(args, action.dest)

            # Add 'global' arguments
            for kw in global_keywords:
                if hasattr(args, kw):
                    subgroup_args[kw] = getattr(args, kw)

            args.subgroup_args[subgroup_title] = subgroup_args

        return args


def create_basic_parser(*args, **kwargs):
    parser = ArgumentParser(*args, **kwargs)
    parser.add_argument(
        "-v",
        "--verbose",
        dest="verbosity",
        action="count",
        default=0,
        help="each occurrence increases verbosity 1 level through ERROR-WARNING-INFO-DEBUG-TRACE (default INFO)",
    )
    parser.add_argument("-l", "--log", dest="log_fn", default=None, help="specify the log filename")
    parser.add_argument(
        "--debug",
        dest="keep_intermediate",
        default=False,
        action="store_true",
        help="Keep intermediate files for future use.",
    )
    parser.add_argument(
        "--overwrite",
        dest="overwrite_existing",
        action="store_true",
        help="Overwrite intermediate or output files if they exist already",
    )
    parser.add_argument(
        "--exit-on-error",
        dest="exit_on_error",
        action="store_true",
        help="exit on first error including non-fatal errors",
    )
    return parser



# Copyright (C) 2023 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""The FY-3E MERSI-LL Level 1B reader operates on Level 1B (L1B) HDF5 files.

The files come in four varieties; band data and geolocation data, both at 250m
and 1000m resolution. Files usually have the following naming scheme:

    tf{start_time:%Y%j%H%M%S}.FY3E-X_MERSI_1000M_L1B.{ext}

This reader's default resampling algorithm is ``ewa`` for Elliptical Weighted
Averaging resampling. The ``--weight-delta-max`` parameter is set to 40 and the
``--weight-distance-max`` parameter is set to 1.

The frontend can be specified with the ``polar2grid.sh`` command using
the ``mersi_ll_l1b`` frontend name. The MERSI-LL frontend provides the following products:

+---------------------------+-----------------------------------------------------+-------------------------+
| **Product Name**          | **Description**                                     | Central Wavelength (um) |
+===========================+=====================================================+=========================+
| 1                         | Channel 1 Day/Night Radiance Band                   | 0.709                   |
+---------------------------+-----------------------------------------------------+-------------------------+
| 2                         | Channel 2 Brightness Temperature Band               | 3.80                    |
+---------------------------+-----------------------------------------------------+-------------------------+
| 3                         | Channel 3 Brightness Temperature Band               | 4.05                    |
+---------------------------+-----------------------------------------------------+-------------------------+
| 4                         | Channel 4 Brightness Temperature Band               | 7.20                    |
+---------------------------+-----------------------------------------------------+-------------------------+
| 5                         | Channel 5 Brightness Temperature Band               | 8.55                    |
+---------------------------+-----------------------------------------------------+-------------------------+
| 6                         | Channel 6 Brightness Temperature Band               | 10.8                    |
+---------------------------+-----------------------------------------------------+-------------------------+
| 7                         | Channel 7 Brightness Temperature Band               | 12.0                    |
+---------------------------+-----------------------------------------------------+-------------------------+
| histogram_dnb             | Histogram Equalized DNB Band                        | N/A                     |
+---------------------------+-----------------------------------------------------+-------------------------+
| adaptive_dnb              | Adaptive Histogram Equalized DNB Band               | N/A                     |
+---------------------------+-----------------------------------------------------+-------------------------+

"""

from __future__ import annotations

from argparse import ArgumentParser, _ArgumentGroup
from typing import Optional

from ._base import ReaderProxyBase

ALL_BANDS = [str(x) for x in range(1, 8)]
ALL_ANGLES = [
    "solar_zenith_angle",
    "solar_azimuth_angle",
    "sensor_zenith_angle",
    "sensor_azimuth_angle",
    "moon_zenith_angle",
    "moon_azimuth_angle",
]
ALL_COMPS = ["histogram_dnb", "adaptive_dnb"]

# band 1 (DNB) is not useful by itself, histogram_dnb is not a default
DEFAULT_PRODUCTS = ALL_BANDS[1:] + ALL_COMPS[1:]

PRODUCT_ALIASES = {}

FILTERS = {
    "day_only": {
        "standard_name": [
            "toa_bidirectional_reflectance",
            "true_color",
            "false_color",
            "natural_color",
        ],
    },
}


class ReaderProxy(ReaderProxyBase):
    """Provide Polar2Grid-specific information about this reader's products."""

    is_polar2grid_reader = True

    def get_default_products(self) -> list[str]:
        """Get products to load if users hasn't specified any others."""
        return DEFAULT_PRODUCTS

    def get_all_products(self) -> list[str]:
        """Get all polar2grid products that could be loaded."""
        return ALL_BANDS + ALL_COMPS + ALL_ANGLES

    @property
    def _aliases(self) -> dict:
        return PRODUCT_ALIASES


def add_reader_argument_groups(
    parser: ArgumentParser, group: Optional[_ArgumentGroup] = None
) -> tuple[Optional[_ArgumentGroup], Optional[_ArgumentGroup]]:
    """Add reader-specific command line arguments to an existing argument parser."""
    return None, None



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2025 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Enhancement functions for VIIRS."""

import numpy as np
from trollimage.xrimage import XRImage
from satpy.enhancements.wrappers import using_map_blocks


def cloud_layers_lmh(img: XRImage) -> XRImage:
    """Convert bit-mask CloudLayer to Low-Medium-High categories.

    This is done in the CIRA version of the product for easier readability.

    Data is a bit-mask of (0=clear), 1 first layer, 2 second layer, 4 third
    layer, 8 fourth layer, 16 fifth layer. The first and second layers are
    grouped into a "Low" layer, third and fourth layers are grouped into
    a "Mid" layer, and the fifth layer is called "High". This function groups
    these 3 layer types with the final output being
    Clear (0), Low (1), Mid (2), Mid + Low (3), High (4), High + Low (5),
    High + Mid (6), and High + Mid + Low (7).

    """
    return _cloud_layers_lmh(img.data)


@using_map_blocks
def _cloud_layers_lmh(data_np: np.ndarray) -> np.ndarray:
    data_out = np.zeros_like(data_np)
    data_out[data_out & 0b00011] += 1  # Low
    data_out[data_out & 0b01100] += 2  # Mid
    data_out[data_out & 0b10000] += 4  # High
    return data_out



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2022 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Test fixtures representing VIIRS swath data."""

from __future__ import annotations

import dask.array as da
import numpy as np
import pytest
import xarray as xr
from pyresample import SwathDefinition
from satpy import Scene
from satpy.tests.utils import make_dataid

from ._fixture_utils import START_TIME, _TestingScene, generate_lonlat_data

VIIRS_I_CHUNKS = (32 * 3, 6400)
VIIRS_M_CHUNKS = (16 * 3, 3200)
VIIRS_DNB_CHUNKS = (16 * 3, 4064)

_mods = ("sunz_corrected_iband",)
VIIRS_I_IDS = [
    make_dataid(name="I01", wavelength=(0.6, 0.64, 0.68), resolution=371, calibration="reflectance", modifiers=_mods),
    make_dataid(
        name="I02", wavelength=(0.845, 0.865, 0.884), resolution=371, calibration="reflectance", modifiers=_mods
    ),
    make_dataid(
        name="I03", wavelength=(1.580, 1.610, 1.640), resolution=371, calibration="reflectance", modifiers=_mods
    ),
    make_dataid(name="I04", wavelength=(3.58, 3.74, 3.90), resolution=371, calibration="brightness_temperature"),
    make_dataid(name="I05", wavelength=(10.5, 11.45, 12.3), resolution=371, calibration="brightness_temperature"),
]
VIIRS_M_IDS = []
_mods = ("sunz_corrected",)
VIIRS_M_IDS.append(
    make_dataid(
        name="M01", wavelength=(0.402, 0.412, 0.422), resolution=742, calibration="reflectance", modifiers=_mods
    )
)
VIIRS_M_IDS.append(
    make_dataid(
        name="M02", wavelength=(0.436, 0.445, 0.454), resolution=742, calibration="reflectance", modifiers=_mods
    )
)
VIIRS_M_IDS.append(
    make_dataid(
        name="M03", wavelength=(0.478, 0.488, 0.498), resolution=742, calibration="reflectance", modifiers=_mods
    )
)
VIIRS_M_IDS.append(
    make_dataid(
        name="M04", wavelength=(0.545, 0.555, 0.565), resolution=742, calibration="reflectance", modifiers=_mods
    )
)
VIIRS_M_IDS.append(
    make_dataid(
        name="M05", wavelength=(0.662, 0.672, 0.682), resolution=742, calibration="reflectance", modifiers=_mods
    )
)
VIIRS_M_IDS.append(
    make_dataid(
        name="M06", wavelength=(0.739, 0.746, 0.754), resolution=742, calibration="reflectance", modifiers=_mods
    )
)
VIIRS_M_IDS.append(
    make_dataid(
        name="M07", wavelength=(0.846, 0.865, 0.885), resolution=742, calibration="reflectance", modifiers=_mods
    )
)
VIIRS_M_IDS.append(
    make_dataid(name="M08", wavelength=(1.23, 1.24, 1.25), resolution=742, calibration="reflectance", modifiers=_mods)
)
VIIRS_M_IDS.append(
    make_dataid(
        name="M09", wavelength=(1.371, 1.378, 1.386), resolution=742, calibration="reflectance", modifiers=_mods
    )
)
VIIRS_M_IDS.append(
    make_dataid(name="M10", wavelength=(1.58, 1.61, 1.64), resolution=742, calibration="reflectance", modifiers=_mods)
)
VIIRS_M_IDS.append(
    make_dataid(name="M11", wavelength=(2.225, 2.25, 2.275), resolution=742, calibration="reflectance", modifiers=_mods)
)
VIIRS_M_IDS.append(
    make_dataid(name="M12", wavelength=(3.61, 3.7, 3.79), resolution=742, calibration="brightness_temperature")
)
VIIRS_M_IDS.append(
    make_dataid(name="M13", wavelength=(3.973, 4.05, 4.128), resolution=742, calibration="brightness_temperature")
)
VIIRS_M_IDS.append(
    make_dataid(name="M14", wavelength=(8.4, 8.55, 8.7), resolution=742, calibration="brightness_temperature")
)
VIIRS_M_IDS.append(
    make_dataid(name="M15", wavelength=(10.263, 10.763, 11.263), resolution=742, calibration="brightness_temperature")
)
VIIRS_M_IDS.append(
    make_dataid(name="M16", wavelength=(11.538, 12.013, 12.489), resolution=742, calibration="brightness_temperature")
)
VIIRS_DNB_IDS = [make_dataid(name="DNB", resolution=743, calibration="radiance")]
VIIRS_M_ANGLES_IDS = []
VIIRS_I_ANGLES_IDS = []
VIIRS_DNB_ANGLES_IDS = []
for angle_suffix in ["solar_zenith_angle", "solar_azimuth_angle", "satellite_zenith_angle", "satellite_azimuth_angle"]:
    VIIRS_M_ANGLES_IDS.append(make_dataid(name=angle_suffix, resolution=742))
    VIIRS_I_ANGLES_IDS.append(make_dataid(name=angle_suffix, resolution=371))
    VIIRS_DNB_ANGLES_IDS.append(make_dataid(name="dnb_" + angle_suffix, resolution=743))
VIIRS_DNB_ANGLES_IDS.append(make_dataid(name="dnb_lunar_zenith_angle", resolution=743))
VIIRS_DNB_ANGLES_IDS.append(make_dataid(name="dnb_lunar_azimuth_angle", resolution=743))
VIIRS_DNB_ANGLES_IDS.append(make_dataid(name="dnb_moon_illumination_fraction"))

# VIIRS_COMP_IDS = [
#     make_dataid(name="dynamic_dnb"),
#     make_dataid(name="histogram_dnb"),
#     make_dataid(name="adaptive_dnb"),
#     make_dataid(name="hncc_dnb"),
#     make_dataid(name="ifog"),
#     make_dataid(name="true_color"),
#     make_dataid(name="false_color"),
# ]
VIIRS_ALL_IDS = (
    VIIRS_I_IDS + VIIRS_M_IDS + VIIRS_DNB_IDS + VIIRS_M_ANGLES_IDS + VIIRS_I_ANGLES_IDS + VIIRS_DNB_ANGLES_IDS
)


@pytest.fixture(scope="session")
def viirs_sdr_i_swath_def() -> SwathDefinition:
    lons, lats = generate_lonlat_data((1536, 6400))
    lons_data_arr = xr.DataArray(
        da.from_array(lons, chunks=VIIRS_I_CHUNKS),
        dims=("y", "x"),
        attrs={
            "rows_per_scan": 32,
            "platform_name": "npp",
            "sensor": "viirs",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "resolution": 371,
            "reader": "viirs_sdr",
        },
    )
    lats_data_arr = xr.DataArray(
        da.from_array(lats, chunks=VIIRS_I_CHUNKS),
        dims=("y", "x"),
        attrs={
            "rows_per_scan": 32,
            "platform_name": "npp",
            "sensor": "viirs",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "resolution": 371,
            "reader": "viirs_sdr",
        },
    )
    return SwathDefinition(lons_data_arr, lats_data_arr)


@pytest.fixture(scope="session")
def viirs_sdr_m_swath_def() -> SwathDefinition:
    lons, lats = generate_lonlat_data((768, 3200))
    lons_data_arr = xr.DataArray(
        da.from_array(lons, chunks=VIIRS_M_CHUNKS),
        dims=("y", "x"),
        attrs={
            "rows_per_scan": 16,
            "platform_name": "npp",
            "sensor": "viirs",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "resolution": 742,
            "reader": "viirs_sdr",
        },
    )
    lats_data_arr = xr.DataArray(
        da.from_array(lats, chunks=VIIRS_M_CHUNKS),
        dims=("y", "x"),
        attrs={
            "rows_per_scan": 16,
            "platform_name": "npp",
            "sensor": "viirs",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "resolution": 742,
            "reader": "viirs_sdr",
        },
    )
    return SwathDefinition(lons_data_arr, lats_data_arr)


@pytest.fixture(scope="session")
def viirs_sdr_dnb_swath_def() -> SwathDefinition:
    lons, lats = generate_lonlat_data((768, 4064))
    lons_data_arr = xr.DataArray(
        da.from_array(lons, chunks=VIIRS_DNB_CHUNKS),
        dims=("y", "x"),
        attrs={
            "rows_per_scan": 16,
            "platform_name": "npp",
            "sensor": "viirs",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "resolution": 743,
            "reader": "viirs_sdr",
        },
    )
    lats_data_arr = xr.DataArray(
        da.from_array(lats, chunks=VIIRS_DNB_CHUNKS),
        dims=("y", "x"),
        attrs={
            "rows_per_scan": 16,
            "platform_name": "npp",
            "sensor": "viirs",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "resolution": 743,
            "reader": "viirs_sdr",
        },
    )
    return SwathDefinition(lons_data_arr, lats_data_arr)


@pytest.fixture
def viirs_sdr_i01_data_array(viirs_sdr_i_swath_def) -> xr.DataArray:
    return xr.DataArray(
        da.zeros((1536, 6400), dtype=np.float32),
        dims=("y", "x"),
        attrs={
            "area": viirs_sdr_i_swath_def,
            "rows_per_scan": 32,
            "platform_name": "npp",
            "sensor": "viirs",
            "name": "I01",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "resolution": 371,
            "reader": "viirs_sdr",
            "calibration": "reflectance",
            "standard_name": "toa_bidirectional_reflectance",
            "units": "%",
        },
    )


@pytest.fixture
def viirs_sdr_i04_data_array(viirs_sdr_i_swath_def) -> xr.DataArray:
    return xr.DataArray(
        da.zeros((1536, 6400), dtype=np.float32),
        dims=("y", "x"),
        attrs={
            "area": viirs_sdr_i_swath_def,
            "rows_per_scan": 32,
            "platform_name": "npp",
            "sensor": "viirs",
            "name": "I04",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "resolution": 371,
            "reader": "viirs_sdr",
            "calibration": "brightness_temperature",
            "standard_name": "toa_brightness_temperature",
            "units": "K",
        },
    )


@pytest.fixture
def viirs_sdr_m01_data_array(viirs_sdr_m_swath_def) -> xr.DataArray:
    return xr.DataArray(
        da.zeros((768, 3200), dtype=np.float32),
        dims=("y", "x"),
        attrs={
            "area": viirs_sdr_m_swath_def,
            "rows_per_scan": 16,
            "platform_name": "npp",
            "sensor": "viirs",
            "name": "M01",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "resolution": 742,
            "reader": "viirs_sdr",
            "calibration": "reflectance",
            "standard_name": "toa_bidirectional_reflectance",
            "units": "%",
        },
    )


@pytest.fixture
def viirs_sdr_m12_data_array(viirs_sdr_m_swath_def) -> xr.DataArray:
    return xr.DataArray(
        da.zeros((768, 3200), dtype=np.float32),
        dims=("y", "x"),
        attrs={
            "area": viirs_sdr_m_swath_def,
            "rows_per_scan": 16,
            "platform_name": "npp",
            "sensor": "viirs",
            "name": "M12",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "resolution": 743,
            "reader": "viirs_sdr",
            "calibration": "brightness_temperature",
            "standard_name": "toa_brightness_temperature",
            "units": "K",
        },
    )


@pytest.fixture
def viirs_sdr_dnb_data_array(viirs_sdr_dnb_swath_def) -> xr.DataArray:
    return xr.DataArray(
        da.zeros((768, 4064), dtype=np.float32),
        dims=("y", "x"),
        attrs={
            "area": viirs_sdr_dnb_swath_def,
            "rows_per_scan": 16,
            "platform_name": "npp",
            "sensor": "viirs",
            "name": "DNB",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "resolution": 743,
            "reader": "viirs_sdr",
            "calibration": "brightness_temperature",
            "standard_name": "toa_brightness_temperature",
            "units": "W m-2 sr-1",
        },
    )


@pytest.fixture
def full_viirs_data_array_dict(
    viirs_sdr_i_dict,
    viirs_sdr_m_dict,
    viirs_sdr_dnb_dict,
):
    data_arrays = {}
    data_arrays.update(viirs_sdr_i_dict)
    data_arrays.update(viirs_sdr_m_dict)
    data_arrays.update(viirs_sdr_dnb_dict)
    return data_arrays


@pytest.fixture
def viirs_sdr_i_dict(viirs_sdr_i01_data_array, viirs_sdr_i04_data_array):
    data_arrays = {}
    for data_id in VIIRS_I_IDS[:3]:
        data_arrays[data_id] = viirs_sdr_i01_data_array.copy()
    for data_id in VIIRS_I_IDS[3:]:
        data_arrays[data_id] = viirs_sdr_i04_data_array.copy()
    data_arrays.update(_viirs_sdr_angles_dict(viirs_sdr_i01_data_array, VIIRS_I_ANGLES_IDS))
    return data_arrays


@pytest.fixture
def viirs_sdr_m_dict(viirs_sdr_m01_data_array, viirs_sdr_m12_data_array):
    data_arrays = {}
    for data_id in VIIRS_M_IDS[:12]:
        data_arrays[data_id] = viirs_sdr_m01_data_array.copy()
    for data_id in VIIRS_M_IDS[12:]:
        data_arrays[data_id] = viirs_sdr_m12_data_array.copy()
    data_arrays.update(_viirs_sdr_angles_dict(viirs_sdr_m01_data_array, VIIRS_M_ANGLES_IDS))
    return data_arrays


@pytest.fixture
def viirs_sdr_dnb_dict(viirs_sdr_dnb_data_array):
    data_arrays = {}
    for data_id in VIIRS_DNB_IDS:
        data_arrays[data_id] = viirs_sdr_dnb_data_array.copy()
    data_arrays.update(_viirs_sdr_angles_dict(viirs_sdr_dnb_data_array, VIIRS_DNB_ANGLES_IDS))
    return data_arrays


def _viirs_sdr_angles_dict(
    base_data_arr,
    angle_ids,
):
    data_arrays = {}
    for angle_id in angle_ids:
        data_arr = base_data_arr.copy()
        del data_arr.attrs["calibration"]
        data_arr.attrs["name"] = angle_id["name"]
        data_arrays[angle_id] = data_arr
    return data_arrays


@pytest.fixture
def viirs_sdr_i01_scene(viirs_sdr_i01_data_array) -> Scene:
    scn = _TestingScene(
        reader="viirs_sdr",
        filenames=["/fake/filename"],
        data_array_dict={
            VIIRS_I_IDS[0]: viirs_sdr_i01_data_array.copy(),
        },
        all_dataset_ids=VIIRS_ALL_IDS,
        available_dataset_ids=VIIRS_I_IDS[:1],
    )
    return scn


@pytest.fixture
def viirs_sdr_full_scene(full_viirs_data_array_dict) -> Scene:
    data_arrays = full_viirs_data_array_dict
    scn = _TestingScene(
        reader="viirs_sdr",
        filenames=["/fake/filename"],
        data_array_dict=data_arrays,
        all_dataset_ids=VIIRS_ALL_IDS,
        available_dataset_ids=list(data_arrays.keys()),
    )
    return scn



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2022 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Test fixtures representing ABI gridded data."""

from __future__ import annotations

import pytest
import xarray as xr
from dask import array as da
from pyresample.geometry import AreaDefinition
from satpy import Scene
from satpy.tests.utils import make_dataid

from ._fixture_utils import START_TIME, _TestingScene


@pytest.fixture(scope="session")
def goes_east_conus_area_def() -> AreaDefinition:
    return AreaDefinition(
        "goes_east",
        "",
        "",
        "+proj=geos +lon_0=-75.0 +h=35786023.0 +a=6378137.0 +b=6356752.31414 +sweep=x +units=m +no_defs",
        5000,
        3000,
        (-3627271.2913, 1583173.6575, 1382771.9287, 4589199.5895),
    )


@pytest.fixture
def abi_l1b_c01_data_array(goes_east_conus_area_def) -> xr.DataArray:
    return xr.DataArray(
        da.zeros((3000, 5000), chunks=4096),
        dims=("y", "x"),
        attrs={
            "area": goes_east_conus_area_def,
            "platform_name": "goes16",
            "sensor": "abi",
            "name": "C01",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "observation_type": "Rad",
            "standard_name": "toa_bidirectional_reflectance",
            "scene_abbr": "C",
            "resolution": 1000,
            "reader": "abi_l1b",
        },
    )


@pytest.fixture
def abi_l1b_airmass_data_array(goes_east_conus_area_def) -> xr.DataArray:
    return xr.DataArray(
        da.zeros((3, 3000, 5000), chunks=4096),
        coords={"bands": ["R", "G", "B"]},
        dims=("bands", "y", "x"),
        attrs={
            "area": goes_east_conus_area_def,
            "platform_name": "goes16",
            "sensor": "abi",
            "name": "airmass",
            "start_time": START_TIME,
            "end_time": START_TIME,
            "observation_type": "Rad",
            "standard_name": "airmass",
            "scene_abbr": "C",
        },
    )


@pytest.fixture
def abi_l1b_c01_scene(abi_l1b_c01_data_array) -> Scene:
    c01_id = make_dataid(name="C01", calibration="reflectance", resolution=1000)
    data_arrays = {
        c01_id: abi_l1b_c01_data_array,
    }
    scn = _TestingScene(
        reader="abi_l1b",
        filenames=["/fake/filename"],
        data_array_dict=data_arrays,
        all_dataset_ids=[c01_id],
        available_dataset_ids=[c01_id],
    )
    return scn


@pytest.fixture
def abi_l1b_airmass_scene(abi_l1b_airmass_data_array) -> Scene:
    scn = Scene()
    scn["airmass"] = abi_l1b_airmass_data_array
    return scn



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2022 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""The GLM Level 2 Reader operates on Gridded GLM Level 2 NetCDF files
from the GOES-16 (GOES-East) and GOES-17/18 (GOES-West) Geostationary
Lightning Mapper (GLM). The GLM L2 reader works off of the input filenames
to determine if a file is supported by Geo2Grid. Files usually have the
following naming scheme::

    OR_GLM-L2-GLMF-M3_G16_s20192701933000_e20192701934000_c20192701934330.nc

These files can be generated by the CSPP Gridded GLM software which depends
on the python ``glmtools`` library and GLM L2 LCFA inputs. The point data from
these inputs is gridded to the ABI full disk, CONUS, or mesoscale sectors.
The GLM L2 reader supports the products shown in the table below, but should
allow for other products available from the GLM files to be loaded.
The GLM L2 reader can be provided to the main geo2grid.sh script
using the ``-r`` option and the reader name ``glm_l2``.

+---------------------------+-----------------------------------------------------+
| **Product Name**          | **Description**                                     |
+===========================+=====================================================+
| flash_extent_density      | Flash Extent Density                                |
+---------------------------+-----------------------------------------------------+
| group_extent_density      | Group Extent Density                                |
+---------------------------+-----------------------------------------------------+
| flash_centroid_density    | Flash Centroid Density                              |
+---------------------------+-----------------------------------------------------+
| group_centroid_density    | Group Centroid Density                              |
+---------------------------+-----------------------------------------------------+
| average_flash_area        | Average Flash Area                                  |
+---------------------------+-----------------------------------------------------+
| minimum_flash_area        | Minimum Flash Area                                  |
+---------------------------+-----------------------------------------------------+
| average_group_area        | Average Group Area                                  |
+---------------------------+-----------------------------------------------------+
| total_energy              | Total Energy                                        |
+---------------------------+-----------------------------------------------------+

"""

from __future__ import annotations

from argparse import ArgumentParser, _ArgumentGroup
from typing import Optional

from ._base import ReaderProxyBase

READER_PRODUCTS = [
    "flash_extent_density",
    "group_extent_density",
    "flash_centroid_density",
    "group_centroid_density",
    "average_flash_area",
    "minimum_flash_area",
    "average_group_area",
    "total_energy",
]
COMPOSITE_PRODUCTS = []


class ReaderProxy(ReaderProxyBase):
    """Provide Polar2Grid-specific information about this reader's products."""

    is_geo2grid_reader = True

    def get_default_products(self) -> list[str]:
        """Get products to load if users hasn't specified any others."""
        return READER_PRODUCTS + COMPOSITE_PRODUCTS

    def get_all_products(self) -> list[str]:
        """Get all polar2grid products that could be loaded."""
        return READER_PRODUCTS + COMPOSITE_PRODUCTS


def add_reader_argument_groups(
    parser: ArgumentParser, group: Optional[_ArgumentGroup] = None
) -> tuple[Optional[_ArgumentGroup], Optional[_ArgumentGroup]]:
    """Add reader-specific command line arguments to an existing argument parser.

    If ``group`` is provided then arguments are added to this group. If not,
    a new group is added to the parser and arguments added to this new group.

    """
    if group is None:
        group = parser.add_argument_group(title="ABI L1b Reader")
    return group, None



#!/usr/bin/env python
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Composites that apply enhancements during the compositing phase."""

from satpy.composites.core import SingleBandCompositor, enhance2dataset


class SingleEnhancedBandCompositor(SingleBandCompositor):
    """Produce a pre-enhanced version of the single provided dependency.

    .. warning::

        This does **NOT** stop Satpy from enhancing this dataset again. The
        metadata provided to this compositor must make the DataArray match
        another "no-op" enhancement on the backend.

    """

    def __call__(self, projectables, nonprojectables=None, **attrs):
        """Build the composite."""
        if len(projectables) != 1:
            raise ValueError("Can't have more than one band in a single-band composite")

        data = projectables[0]
        new_attrs = data.attrs.copy()
        data = enhance2dataset(data)
        data.attrs = new_attrs
        if isinstance(data.attrs.get("sensor"), set) and len(data.attrs["sensor"]) == 1:
            data.attrs["sensor"] = list(data.attrs["sensor"])[0]
        return super().__call__([data], **attrs)


class SubBandCompositor(SingleBandCompositor):
    """Get a single band from a provided multi-band DataArray."""

    def __init__(self, name, prerequisites=None, optional_prerequisites=None, band_index=None, **kwargs):
        super().__init__(name, prerequisites=prerequisites, optional_prerequisites=optional_prerequisites, **kwargs)
        if band_index is None:
            raise ValueError("'band_index' must be provided.")
        self._band_index = band_index

    def __call__(self, projectables, nonprojectables=None, **attrs):
        """Index the provided DataArray and create a single band composite from it."""
        if len(projectables) != 1:
            raise ValueError("Can't have more than one band in a single-band composite")

        data = projectables[0]
        if "bands" not in data.dims:
            raise ValueError("Provided data has no 'bands' dimension.")

        band_index = self._band_index
        if isinstance(band_index, int):
            band_index = data.dims["bands"][band_index]
        data = data.sel(bands=band_index)
        return super().__call__([data], **attrs)



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2017-2021 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""The ACSPO reader is for reading files created by the NOAA Community
Satellite Processing Package (CSPP) Advanced Clear-Sky
Processor for Oceans (ACSPO) system software. The ACSPO reader supports
product files created from VIIRS, MODIS and AVHRR imaging sensors.
For more information on this product, please visit the CSPP LEO
website: `https://cimss.ssec.wisc.edu/cspp/`.

The ACSPO output product format is NetCDF4.  The frontend can be specified
with the ``polar2grid.sh`` command using the ``acspo`` frontend name.
The ACSPO frontend provides the following products:

+------------------------+--------------------------------------------+
| Product Name           | Description                                |
+========================+============================================+
| sst                    | Sea Surface Temperature                    |
+------------------------+--------------------------------------------+

"""

from __future__ import annotations

from argparse import ArgumentParser, _ArgumentGroup
from typing import Optional

from satpy import DataQuery

from ._base import ReaderProxyBase
from ..core.script_utils import BooleanFilterAction

DEFAULT_PRODUCTS = ["sst"]
PREFERRED_CHUNK_SIZE = 2048


class ReaderProxy(ReaderProxyBase):
    """Provide Polar2Grid-specific information about this reader's products."""

    is_polar2grid_reader = True

    def get_default_products(self) -> list[str]:
        """Get products to load if users hasn't specified any others."""
        return DEFAULT_PRODUCTS

    def get_all_products(self) -> list[str]:
        """Get all polar2grid products that could be loaded."""
        return DEFAULT_PRODUCTS

    @property
    def _aliases(self) -> dict[str, DataQuery]:
        return {}


def add_reader_argument_groups(
    parser: ArgumentParser, group: Optional[_ArgumentGroup] = None
) -> tuple[Optional[_ArgumentGroup], Optional[_ArgumentGroup]]:
    """Add reader-specific command line arguments to an existing argument parser.

    If ``group`` is provided then arguments are added to this group. If not,
    a new group is added to the parser and arguments added to this new group.

    """
    if group is None:
        group = parser.add_argument_group(title="ACSPO Reader")
    group.add_argument(
        "--cloud-clear",
        action=BooleanFilterAction,
        dest="filters",
        const="cloud_clear",
        default=["cloud_clear"],
        help="Enable or disable cloud clearing for the 'sst' product (default on).",
    )
    return group, None



#!/usr/bin/env python3
# encoding: utf-8
"""Convert GEOCAT Level 1 and 2 products to AWIPS SCMI format.

Run with the following command from the Polar2Grid Software Bundle:

    source /path/to/polar2grid/bin/env.sh
    python geocat2scmi.py ...

Use `-h` flag for help and usage information.

FUTURE: This script will be included in the functionality of the CSPP Geo2Grid package

"""

import logging
import sys


def add_scene_argument_groups(parser):
    group_1 = parser.add_argument_group(title="Scene Initialization")
    group_1.add_argument("reader", help="Name of reader used to read provided files")
    group_1.add_argument("-f", "--filenames", nargs="+", help="Input files to read")
    group_2 = parser.add_argument_group(title="Scene Load")
    group_2.add_argument("-d", "--datasets", nargs="+", help="Names of datasets to load from input files")
    return group_1, group_2


def main():
    import argparse

    from satpy import Scene

    from polar2grid.writers.awips_tiled import add_writer_argument_groups

    parser = argparse.ArgumentParser(description="Convert GEOCAT Level 1 and 2 to AWIPS SCMI files")
    parser.add_argument(
        "-v",
        "--verbose",
        dest="verbosity",
        action="count",
        default=0,
        help="each occurrence increases verbosity 1 level through ERROR-WARNING-INFO-DEBUG (default INFO)",
    )
    parser.add_argument("-l", "--log", dest="log_fn", default=None, help="specify the log filename")
    subgroups = add_scene_argument_groups(parser)
    subgroups += add_writer_argument_groups(parser)
    args = parser.parse_args()

    scene_args = {ga.dest: getattr(args, ga.dest) for ga in subgroups[0]._group_actions}
    load_args = {ga.dest: getattr(args, ga.dest) for ga in subgroups[1]._group_actions}
    writer_init_args = {ga.dest: getattr(args, ga.dest) for ga in subgroups[2]._group_actions}
    writer_call_args = {ga.dest: getattr(args, ga.dest) for ga in subgroups[3]._group_actions}

    levels = [logging.ERROR, logging.WARN, logging.INFO, logging.DEBUG]
    logging.basicConfig(level=levels[min(3, args.verbosity)], filename=args.log_fn)

    scn = Scene(**scene_args)
    scn.load(load_args["datasets"])
    writer_args = {}
    writer_args.update(writer_init_args)
    writer_args.update(writer_call_args)
    scn.save_datasets(writer="scmi", **writer_args)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2018 Space Science and Engineering Center (SSEC),
#  University of Wisconsin-Madison.
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
#
#     Written by David Hoese    March 2016
#     University of Wisconsin-Madison
#     Space Science and Engineering Center
#     1225 West Dayton Street
#     Madison, WI  53706
#     david.hoese@ssec.wisc.edu
"""The GeoTIFF writer puts gridded image data into a standard GeoTIFF file.

It uses the GDAL python API and rasterio python package to create the GeoTIFF files.
It can handle any grid that can be described by PROJ.4 and understood by the
GeoTIFF format.

By default the 'geotiff' writer will add an "Alpha" band to the file to mark
any invalid or missing data pixels. This results in invalid pixels showing up
as transparent in most image viewers.

"""

import argparse
import logging
import os
from argparse import BooleanOptionalAction

from polar2grid.core.dtype import NUMPY_DTYPE_STRS, int_or_float, str_to_dtype
from polar2grid.core.script_utils import NumpyDtypeList
from polar2grid.utils.legacy_compat import convert_p2g_pattern_to_satpy

LOG = logging.getLogger(__name__)

# reader_name -> filename
DEFAULT_OUTPUT_FILENAMES = {
    "polar2grid": {
        None: "{platform_name!l}_{sensor!l}_{p2g_name}_{start_time:%Y%m%d_%H%M%S}_{area.area_id}.tif",
    },
    "geo2grid": {
        "abi_l1b": "{platform_name!u}_{sensor!u}_{observation_type}{scene_abbr}_"
        "{p2g_name}_{start_time:%Y%m%d_%H%M%S}_{area.area_id}.tif",
        "clavrx": "{platform_name!l}_{sensor!l}_{p2g_name}_{start_time:%Y%m%d_%H%M%S}_{area.area_id}.tif",
        None: "{platform_name!u}_{sensor!u}_{p2g_name}_{start_time:%Y%m%d_%H%M%S}_{area.area_id}.tif",
    },
}


def add_writer_argument_groups(parser, group=None):
    from argparse import SUPPRESS

    if group is None:
        group = parser.add_argument_group(title="Geotiff Writer")
    group.add_argument(
        "--output-filename",
        dest="filename",
        type=convert_p2g_pattern_to_satpy,
        help="Custom file pattern to save dataset to",
    )
    group.add_argument(
        "--dtype",
        choices=NumpyDtypeList(NUMPY_DTYPE_STRS),
        type=str_to_dtype,
        help="Data type of the output file (8-bit unsigned integer by default - uint8)",
    )
    group.add_argument(
        "--no-enhance",
        dest="enhance",
        action="store_false",
        help="Don't try to enhance the data before saving it",
    )
    group.add_argument(
        "--fill-value",
        dest="fill_value",
        type=int_or_float,
        help="Instead of an alpha channel fill invalid "
        "values with this value. Turns LA or RGBA "
        "images in to L or RGB images respectively.",
    )
    group.add_argument(
        "--compress",
        default="LZW",
        help="File compression algorithm (DEFLATE, LZW, NONE, etc)",
    )
    group.add_argument(
        "--keep-palette",
        action="store_true",
        help="When saving 'palettized' enhanced images, save the colormap as a "
        "geotiff color table instead of converting the image to RGB/A",
    )
    group.add_argument(
        "--tiled", action=BooleanOptionalAction, default=True, help="Tile geotiffs internally (default: True)"
    )
    group.add_argument("--blockxsize", default=SUPPRESS, type=int, help="Set tile block X size")
    group.add_argument("--blockysize", default=SUPPRESS, type=int, help="Set tile block Y size")
    group.add_argument(
        "--scale-offset-tags",
        default=["scale", "offset"],
        type=_check_tags,
        # help="Specify custom geotiff tags for enhancement metadata. Should be "
        # "two comma-separated names for the metadata tags. Defaults to "
        # "'scale,offset'. Specify 'NONE' to not save the tags.",
        help=SUPPRESS,
    )
    group.add_argument(
        "--colormap-tag",
        default="colormap",
        type=lambda input_str: None if input_str == "NONE" else input_str,
        # help="Specify the custom geotiff tag where a CSV version of an applied " "colormap (if any) will be saved.",
        help=SUPPRESS,
    )
    group.add_argument(
        "--gdal-num-threads",
        dest="num_threads",
        default=os.environ.get("DASK_NUM_WORKERS", 4),
        help=SUPPRESS,
    )  # don't show this option to the user
    # help='Set number of threads used for compressing '
    #      'geotiffs (default: Same as num-workers)')
    group.add_argument(
        "--overviews",
        type=lambda x: x.split(" "),
        help="Build lower resolution versions of your image "
        "for better performance in some clients. "
        "Specified as a space separate list of numbers, "
        "typically as powers of 2. Example: '2 4 8 16'",
    )
    group.add_argument(
        "--overviews-resampling",
        default="nearest",
        choices=("nearest", "average", "bilinear", "cubic", "cubicspline", "lanczos"),
        help="Specify resampling used when generating overviews",
    )
    group.add_argument(
        "--gdal-driver",
        dest="driver",
        help="Name of the GDAL driver to use when writing the geotiff. "
        "By default the 'geotiff' driver is used. If '--driver COG' "
        "is used then the GDAL 'COG' driver will be used and will create "
        "a tiled COG-compatible geotiff.",
    )
    # Saving specific keyword arguments
    # group_2 = parser.add_argument_group(title='Writer Save')
    return group, None


def _check_tags(input_str):
    if "none" in input_str.lower():
        return None
    split_names = input_str.split(",")
    if len(split_names) != 2:
        raise argparse.ArgumentError("Expected 2 comma-separated names or 'NONE'.")
    return split_names



#!/usr/bin/env python3
# encoding: utf-8
# Copyright (C) 2021 Space Science and Engineering Center (SSEC),
# University of Wisconsin-Madison.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This file is part of the polar2grid software package. Polar2grid takes
# satellite observation data, remaps it, and writes it to a file format for
# input into another program.
# Documentation: http://www.ssec.wisc.edu/software/polar2grid/
"""Tests for the convert_grids_conf_to_yaml.py script."""

from io import StringIO

import pytest
from pyresample import parse_area_file
from pyresample.geometry import AreaDefinition, DynamicAreaDefinition


@pytest.mark.parametrize(
    ("conf_content", "num_areas", "area_types"),
    [
        (
            """
wgs84_fit, proj4, +proj=latlong +datum=WGS84 +no_defs, None, None, 0.0057, -0.0057, None, None
""",
            1,
            [DynamicAreaDefinition],
        ),
        (
            """
211e, proj4, +proj=lcc +a=6371200 +b=6371200 +lat_0=25 +lat_1=25 +lon_0=-95 +units=m +no_defs, 5120, 5120, 1015.9, -1015.9, -123.044deg, 59.844deg
211e_10km, proj4, +proj=lcc +a=6371200 +b=6371200 +lat_0=25 +lat_1=25 +lon_0=-95 +units=m +no_defs, 512, 512, 10159, -10159, -123.044deg, 59.844deg
211e_hi, proj4, +proj=lcc +a=6371200 +b=6371200 +lat_0=25 +lat_1=25 +lon_0=-95 +units=m +no_defs, 10000, 10000, 500, -500, -123.044deg, 59.844deg
""",  # noqa
            3,
            [AreaDefinition] * 3,
        ),
    ],
)
def test_conf_conversion(tmpdir, capsys, conf_content, num_areas, area_types):
    from polar2grid.utils.convert_grids_conf_to_yaml import main

    grids_fn = tmpdir.join("test_grids.conf")
    with open(grids_fn, "w") as grids_file:
        grids_file.write(conf_content)

    main([str(grids_fn)])
    sout = capsys.readouterr()
    s = StringIO()
    s.write(sout.out)
    s.seek(0)
    areas = parse_area_file([s])
    assert len(areas) == num_areas
    for area_obj, area_type in zip(areas, area_types, strict=True):
        assert isinstance(area_obj, area_type)



"""Warnings or utilities for dealing with warnings."""

from __future__ import annotations

import contextlib
import warnings


@contextlib.contextmanager
def ignore_no_georef():
    """Wrap operations that we know will produce a rasterio geolocation warning."""
    from rasterio.errors import NotGeoreferencedWarning

    with warnings.catch_warnings():
        warnings.filterwarnings(
            "ignore",
            "Dataset has no geotransform",
            NotGeoreferencedWarning,
        )
        yield


@contextlib.contextmanager
def ignore_pyproj_proj_warnings():
    """Wrap operations that we know will produce a PROJ.4 precision warning.

    Only to be used internally to Pyresample when we have no other choice but
    to use PROJ.4 strings/dicts. For example, serialization to YAML or other
    human-readable formats or testing the methods that produce the PROJ.4
    versions of the CRS.

    """
    with warnings.catch_warnings():
        warnings.filterwarnings(
            "ignore",
            "You will likely lose important projection information",
            UserWarning,
        )
        yield
